using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Grouping;


namespace NestedTableHide
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class MainForm : System.Windows.Forms.Form
	{
        private Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl WidgetGroupingGrid;
        private System.Windows.Forms.Button SaveButton;

        private WidgetDataSet _WidgetData;
        private ArrayList _NewExpandedRecords;


        /// <summary>
        /// Gets a lazy created instance of the WidgetDataSet and fills it with the default working set of data.
        /// DataColumn constraints are also set here on first create.
        /// </summary>
        private WidgetDataSet WidgetData 
        {
            get 
            {
                if (_WidgetData == null) 
                {
                    _WidgetData = new WidgetDataSet();
                    _WidgetData.EnforceConstraints = true;
                    _WidgetData.Widget.NameColumn.Unique = true;
                    _WidgetData.Widget.NameColumn.AllowDBNull = false;
                    _WidgetData.Widget.WidgetTypeIDColumn.AllowDBNull = false;
                    this.LoadData(_WidgetData);
                }
                return _WidgetData;
            }
        }

        
        private ArrayList NewExpandedRecords 
        {
            get 
            {
                if (_NewExpandedRecords == null) 
                {
                    _NewExpandedRecords = new ArrayList();
                }
                return _NewExpandedRecords;
            }
        }

        
        /// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public MainForm()
		{
            GridEngineFactory.Factory = new CustomGroupingEngineFactory();

			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
            this.SetupGridColumns();
		}


		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}


		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.WidgetGroupingGrid = new Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl();
            this.SaveButton = new System.Windows.Forms.Button();
            ((System.ComponentModel.ISupportInitialize)(this.WidgetData)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.WidgetGroupingGrid)).BeginInit();
            this.SuspendLayout();
            // 
            // WidgetGroupingGrid
            // 
            this.WidgetGroupingGrid.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
                | System.Windows.Forms.AnchorStyles.Left) 
                | System.Windows.Forms.AnchorStyles.Right)));
            this.WidgetGroupingGrid.BackColor = System.Drawing.SystemColors.Window;
            this.WidgetGroupingGrid.Location = new System.Drawing.Point(8, 8);
            this.WidgetGroupingGrid.Name = "WidgetGroupingGrid";
            this.WidgetGroupingGrid.Size = new System.Drawing.Size(520, 416);
            this.WidgetGroupingGrid.TabIndex = 0;
            this.WidgetGroupingGrid.Text = "gridGroupingControl1";
            this.WidgetGroupingGrid.VersionInfo = "4.101.0.63";
            this.WidgetGroupingGrid.RecordExpanding += new Syncfusion.Grouping.RecordEventHandler(this.WidgetGroupingGrid_RecordExpanding);
            this.WidgetGroupingGrid.TableControlCurrentCellCloseDropDown += new Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlPopupClosedEventHandler(this.WidgetGroupingGrid_TableControlCurrentCellCloseDropDown);
            // 
            // SaveButton
            // 
            this.SaveButton.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this.SaveButton.Location = new System.Drawing.Point(408, 440);
            this.SaveButton.Name = "SaveButton";
            this.SaveButton.Size = new System.Drawing.Size(104, 40);
            this.SaveButton.TabIndex = 1;
            this.SaveButton.Text = "Save";
            this.SaveButton.Click += new System.EventHandler(this.SaveButton_Click);
            // 
            // MainForm
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(544, 485);
            this.Controls.Add(this.SaveButton);
            this.Controls.Add(this.WidgetGroupingGrid);
            this.Name = "MainForm";
            this.Text = "Widget Grouping Example";
            this.Load += new System.EventHandler(this.MainForm_Load);
            ((System.ComponentModel.ISupportInitialize)(this.WidgetData)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.WidgetGroupingGrid)).EndInit();
            this.ResumeLayout(false);

        }

		
        
        /// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new MainForm());
		}



        private void SetupGridColumns() 
        {
            this.SuspendLayout();
            this.WidgetGroupingGrid.CausesValidation = true;

            Syncfusion.Windows.Forms.Grid.Grouping.GridColumnDescriptor gridColumnDescriptor1 = new Syncfusion.Windows.Forms.Grid.Grouping.GridColumnDescriptor();
            Syncfusion.Windows.Forms.Grid.Grouping.GridColumnDescriptor gridColumnDescriptor2 = new Syncfusion.Windows.Forms.Grid.Grouping.GridColumnDescriptor();
            Syncfusion.Windows.Forms.Grid.Grouping.GridColumnDescriptor gridColumnDescriptor3 = new Syncfusion.Windows.Forms.Grid.Grouping.GridColumnDescriptor();
            Syncfusion.Windows.Forms.Grid.Grouping.GridRelationDescriptor gridRelationDescriptor1 = new Syncfusion.Windows.Forms.Grid.Grouping.GridRelationDescriptor();
            Syncfusion.Windows.Forms.Grid.Grouping.GridRelationDescriptor gridRelationDescriptor2 = new Syncfusion.Windows.Forms.Grid.Grouping.GridRelationDescriptor();

            gridColumnDescriptor1.MappingName = "WidgetID";
            gridColumnDescriptor1.ReadOnly = true;
            gridColumnDescriptor1.Width = 68;

            gridColumnDescriptor2.MappingName = "Name";
            gridColumnDescriptor2.Width = 150;
            
            gridColumnDescriptor3.Appearance.AnyRecordFieldCell.CellType = "ComboBox";
            gridColumnDescriptor3.Appearance.AnyRecordFieldCell.CellValueType = typeof(int);
            gridColumnDescriptor3.Appearance.AnyRecordFieldCell.DataSource = this.WidgetData.WidgetType;
            gridColumnDescriptor3.Appearance.AnyRecordFieldCell.DisplayMember = "Name";
            gridColumnDescriptor3.Appearance.AnyRecordFieldCell.DropDownStyle = Syncfusion.Windows.Forms.Grid.GridDropDownStyle.Exclusive;
            gridColumnDescriptor3.Appearance.AnyRecordFieldCell.ValueMember = "WidgetTypeID";
            gridColumnDescriptor3.HeaderText = "Widget Type";
            gridColumnDescriptor3.MappingName = "WidgetTypeID";
            gridColumnDescriptor3.Width = 150;

            this.WidgetGroupingGrid.TableDescriptor.Columns.AddRange(
                new Syncfusion.Windows.Forms.Grid.Grouping.GridColumnDescriptor[] {
                    gridColumnDescriptor1, gridColumnDescriptor2, gridColumnDescriptor3});

            gridRelationDescriptor1.ChildTableDescriptor.TopLevelGroupOptions.ShowAddNewRecordAfterDetails = true;
            gridRelationDescriptor1.ChildTableDescriptor.TopLevelGroupOptions.ShowAddNewRecordBeforeDetails = false;
            gridRelationDescriptor1.MappingName = "FK_Widget_WidgetMaintenanceLog";
            
            gridRelationDescriptor2.ChildTableDescriptor.TopLevelGroupOptions.ShowAddNewRecordAfterDetails = true;
            gridRelationDescriptor2.ChildTableDescriptor.TopLevelGroupOptions.ShowAddNewRecordBeforeDetails = false;
            gridRelationDescriptor2.MappingName = "FK_Widget_WidgetPartsList";
            
            this.WidgetGroupingGrid.TableDescriptor.Relations.AddRange(new Syncfusion.Windows.Forms.Grid.Grouping.GridRelationDescriptor[] {
                gridRelationDescriptor1, gridRelationDescriptor2});

            this.WidgetGroupingGrid.TableDescriptor.TopLevelGroupOptions.CaptionText = "The Widget: {RecordCount} Items";
            this.WidgetGroupingGrid.TableDescriptor.TopLevelGroupOptions.ShowAddNewRecordAfterDetails = true;
            this.WidgetGroupingGrid.TableDescriptor.TopLevelGroupOptions.ShowAddNewRecordBeforeDetails = false;
            
            this.WidgetGroupingGrid.TableDescriptor.VisibleColumns.AddRange(
                new Syncfusion.Windows.Forms.Grid.Grouping.GridVisibleColumnDescriptor[] {
                    new Syncfusion.Windows.Forms.Grid.Grouping.GridVisibleColumnDescriptor("WidgetID"),
                    new Syncfusion.Windows.Forms.Grid.Grouping.GridVisibleColumnDescriptor("Name"),
                    new Syncfusion.Windows.Forms.Grid.Grouping.GridVisibleColumnDescriptor("WidgetTypeID")});

            this.ResumeLayout();
        }



        private void LoadData(WidgetDataSet ds) 
        {
            // NOTE: look in the WidgetData property get to view the constraints being set on the dataset

            // We only want to show the WidgetMaintenanceLog child table for "Industrial Widgets"
            WidgetDataSet.WidgetTypeRow industrialWidgetTypeRow = ds.WidgetType.NewWidgetTypeRow();
            industrialWidgetTypeRow.WidgetTypeID = 1;
            industrialWidgetTypeRow.Name = "Industrial Widget";
            ds.WidgetType.AddWidgetTypeRow(industrialWidgetTypeRow);

            // We DO NOT want to show the WidgetMaintenanceLog for "Toy Widgets"
            WidgetDataSet.WidgetTypeRow toyWidgetTypeRow = ds.WidgetType.NewWidgetTypeRow();
            toyWidgetTypeRow.WidgetTypeID = 2;
            toyWidgetTypeRow.Name = "Toy Widget";
            ds.WidgetType.AddWidgetTypeRow(toyWidgetTypeRow);

            // The user should be able to add maintenance log entries for this Bulldozer
            WidgetDataSet.WidgetRow widgetRow = ds.Widget.NewWidgetRow();
            widgetRow.WidgetID = 1;
            widgetRow.Name = "Bulldozer";
            widgetRow.WidgetTypeRow = industrialWidgetTypeRow;
            ds.Widget.AddWidgetRow(widgetRow);

            // Add some parts for the "Bulldozer" widget object
            WidgetDataSet.WidgetPartsListRow partsRow = ds.WidgetPartsList.NewWidgetPartsListRow();
            partsRow.WidgetPartsListID = 1;
            partsRow.WidgetRow = widgetRow;
            partsRow.Name = "Bulldozer blade";
            ds.WidgetPartsList.AddWidgetPartsListRow(partsRow);

            partsRow = ds.WidgetPartsList.NewWidgetPartsListRow();
            partsRow.WidgetPartsListID = 2;
            partsRow.WidgetRow = widgetRow;
            partsRow.Name = "Diesel Engine";
            ds.WidgetPartsList.AddWidgetPartsListRow(partsRow);


            // The user should NOT be able to add maintenance log entries for this Tonka Truck
            // Futher, we do not even want to show the WidgetMaintenanceLog child table for this item 
            // because its WidgetTypeID is set to 2.
            // However, we do want to show the parts list for the Tonka Truck
            widgetRow = ds.Widget.NewWidgetRow();
            widgetRow.WidgetID = 2;
            widgetRow.Name = "Tonka Truck";
            widgetRow.WidgetTypeRow = toyWidgetTypeRow;
            ds.Widget.AddWidgetRow(widgetRow);

            // Add some parts for the Tonka Truck
            partsRow = ds.WidgetPartsList.NewWidgetPartsListRow();
            partsRow.WidgetPartsListID = 3;
            partsRow.WidgetRow = widgetRow;
            partsRow.Name = "Plastic Wheels";
            ds.WidgetPartsList.AddWidgetPartsListRow(partsRow);

            partsRow = ds.WidgetPartsList.NewWidgetPartsListRow();
            partsRow.WidgetPartsListID = 4;
            partsRow.WidgetRow = widgetRow;
            partsRow.Name = "Yellow Paint";
            ds.WidgetPartsList.AddWidgetPartsListRow(partsRow);

            ds.AcceptChanges();
        }



        private void MainForm_Load(object sender, System.EventArgs e)
        {
            this.WidgetGroupingGrid.DataSource = this.WidgetData.Widget;
        }


		private void WidgetGroupingGrid_RecordExpanding(object sender, Syncfusion.Grouping.RecordEventArgs e)
		{
			if( e.Record != null )
			{
                this.SetChildTableVisibility(e.Record);
			}
		}


		private void SetChildTableVisibility(Record r)
		{
            NestedTablesCollection tables = r.NestedTables;
			foreach( NestedTable table in tables )
			{
				if( table.ChildTable != null && table.ChildTable.Name == "WidgetMaintenanceLog" ) 
				{
					foreach( Element el in table.ChildTable.DisplayElements)
					{
						if( el.ParentChildTable.ParentNestedTable.ParentRecord != null)
						{
                            object obj = el.ParentChildTable.ParentNestedTable.ParentRecord.GetValue("WidgetTypeID"); 
                            
                            // In the real world we would want to pull these row heights from some kind of settings object or enumerated type
                            switch((int)obj) 
                            {
                                case 1 :
                                    ((IGridRowHeight)el).RowHeight = 20;
                                    break;
                                case 2 :
                                    ((IGridRowHeight)el).RowHeight = 0;
                                    break;
                            }
						}
					}
				}
			}
		}


		private void WidgetGroupingGrid_TableControlCurrentCellCloseDropDown(object sender, Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlPopupClosedEventArgs e)
		{
			if(e.Inner.PopupCloseType == Syncfusion.Windows.Forms.PopupCloseType.Done )
			{
				GridTableCellStyleInfo tableCellStyle = e.TableControl.CurrentCell.Renderer.CurrentStyle as GridTableCellStyleInfo ;
				if( tableCellStyle.TableCellIdentity.Column != null &&  tableCellStyle.TableCellIdentity.Column.Name == "WidgetTypeID"  && tableCellStyle.TableCellIdentity.DisplayElement.Kind != DisplayElementKind.AddNewRecord  ) 
				{
					e.TableControl.Table.CurrentRecord.EndEdit();
                    e.TableControl.Table.CurrentRecord.IsExpanded = false;

                    // We need to always reset the visiblity settings of the child table every time a dropdown has been changed
                    this.SetChildTableVisibility(e.TableControl.Table.CurrentRecord);
				}
			}

		}


        private void SaveButton_Click(object sender, System.EventArgs e)
        {
            // 
            // NOTE: We have to update the tables separately as shown below because of we have dependant
            //       parent-child relationships that must be preserved as well as autonumbering keys.
            //       Conventional SqlDataAdapters are not smart enough to do everything in one transaction 
            //       by calling the SqlDataAdapter.Update() method for all DataRow states.
            //

			// need to process the deleted rows first starting with the children first
            this.FakeDeleteDataTable(this.WidgetData.WidgetMaintenanceLog);
            this.FakeDeleteDataTable(this.WidgetData.WidgetPartsList);
            this.FakeDeleteDataTable(this.WidgetData.Widget);

		    // We want to clear out the list of currently expanded new records that could be in there from the last save operation
        	this.NewExpandedRecords.Clear();
            
            // Attach the DisplayElement changing Event so we can identify and save the rows 
            // that have been changed by the fake database update
            this.WidgetGroupingGrid.Table.DisplayElementChanging += new DisplayElementChangingEventHandler(Table_DisplayElementChanging); 

            // insert the rows starting with the parents going down to the children breadth first
            this.FakeInsertDataTable(this.WidgetData.Widget, "WidgetID");
            this.FakeInsertDataTable(this.WidgetData.WidgetMaintenanceLog, "WidgetMaintenanceLogID");
            this.FakeInsertDataTable(this.WidgetData.WidgetPartsList, "WidgetPartsListID");
			
			// We don't need to listen to the DisplayElementChanging event once the insert has completed, so stop listening to it
			this.WidgetGroupingGrid.Table.DisplayElementChanging -= new DisplayElementChangingEventHandler(Table_DisplayElementChanging); 
            
            // Loop through the expanded records and toggle their IsExpanded state on then off again
            // somehow this tells the grid that the child records need to be repainted
            foreach(Record rec in this.NewExpandedRecords) 
            {
                rec.IsExpanded = false;
                rec.IsExpanded = true;
            }

			// update the existing rows...order doesn't matter
            this.FakeUpdateDataTable(this.WidgetData.Widget);
            this.FakeUpdateDataTable(this.WidgetData.WidgetMaintenanceLog);
            this.FakeUpdateDataTable(this.WidgetData.WidgetPartsList);
        }


        private void FakeInsertDataTable(DataTable table, string primaryKeyField) 
        {
            // get the max value if we are dealing with added values
            object maxValue = table.Compute("MAX("+primaryKeyField+")", "");
            int maxPK = -1;
            if (maxValue != null && maxValue != DBNull.Value) 
            {
                maxPK = (int)maxValue;
            }
            int nextPK = 1;
            if (maxPK > 0) 
            {
                nextPK = maxPK+1;
            }

            // renumber the primary keys with positive values like a SQL identity field would
            foreach(DataRow row in table.Select("", "", DataViewRowState.Added)) 
            {
                row[primaryKeyField] = nextPK;
                nextPK++;
                row.AcceptChanges();
            }
        }

        private void FakeDeleteDataTable(DataTable table) 
        {
            foreach(DataRow row in table.Select("", "", DataViewRowState.Deleted)) 
            {
                row.AcceptChanges();
            }
        }

        private void FakeUpdateDataTable(DataTable table) 
        {
            foreach(DataRow row in table.Select("", "", DataViewRowState.ModifiedCurrent)) 
            {
                row.AcceptChanges();
            }
		}

		
		private void Table_DisplayElementChanging(object sender, DisplayElementChangingEventArgs e)
		{
            if( e.Element.ParentTableDescriptor.Name == "Widget") 
            {
                // We only want to toggle on-and-off the Widget table, since it is the one that has conditionally 
                // viewable tables
                Record rec = e.Element as Record;
                if (rec != null && this.NewExpandedRecords.IndexOf(e.Element as Record) < 0) 
                {
                    // We only want to add records that aren't already in the list
                    this.NewExpandedRecords.Add(rec);
                }
            }
		}


	}
}
